@@ -119,6 +119,7 @@ from .model import (
     DeviceInfo,
     EntityInfo,
     EntityState,
+    ESPHomeBluetoothGATTServices,
     FanDirection,
     FanInfo,
     FanSpeed,
@@ -152,6 +153,8 @@ from .model import (
 
 _LOGGER = logging.getLogger(__name__)
 
+DEFAULT_BLE_TIMEOUT = 30.0
+
 ExecuteServiceDataType = Dict[
     str, Union[bool, int, float, str, List[bool], List[int], List[float], List[str]]
 ]
@@ -420,8 +423,8 @@ class APIClient:
         )
 
         def unsub() -> None:
-            assert self._connection is not None
-            self._connection.remove_message_callback(on_msg)
+            if self._connection is not None:
+                self._connection.remove_message_callback(on_msg)
 
         return unsub
 
@@ -441,8 +444,8 @@ class APIClient:
         )
 
         def unsub() -> None:
-            assert self._connection is not None
-            self._connection.remove_message_callback(on_msg)
+            if self._connection is not None:
+                self._connection.remove_message_callback(on_msg)
 
         return unsub
 
@@ -450,7 +453,7 @@ class APIClient:
         self,
         address: int,
         on_bluetooth_connection_state: Callable[[bool, int, int], None],
-        timeout: float = 10.0,
+        timeout: float = DEFAULT_BLE_TIMEOUT,
     ) -> Callable[[], None]:
         self._check_authenticated()
 
@@ -472,15 +475,19 @@ class APIClient:
             on_msg,
         )
 
+        def unsub() -> None:
+            if self._connection is not None:
+                self._connection.remove_message_callback(on_msg)
+
         try:
             async with async_timeout.timeout(timeout):
                 await event.wait()
         except asyncio.TimeoutError as err:
+            unsub()
             raise TimeoutAPIError("Timeout waiting for connect response") from err
-
-        def unsub() -> None:
-            assert self._connection is not None
-            self._connection.remove_message_callback(on_msg)
+        finally:
+            if not event.is_set():
+                await self.bluetooth_device_disconnect(address)
 
         return unsub
 
@@ -495,7 +502,9 @@ class APIClient:
             )
         )
 
-    async def bluetooth_gatt_get_services(self, address: int) -> BluetoothGATTServices:
+    async def bluetooth_gatt_get_services(
+        self, address: int
+    ) -> ESPHomeBluetoothGATTServices:
         self._check_authenticated()
 
         def do_append(msg: message.Message) -> bool:
@@ -506,15 +515,21 @@ class APIClient:
 
         assert self._connection is not None
         resp = await self._connection.send_message_await_response_complex(
-            BluetoothGATTGetServicesRequest(address=address), do_append, do_stop
+            BluetoothGATTGetServicesRequest(address=address),
+            do_append,
+            do_stop,
+            timeout=DEFAULT_BLE_TIMEOUT,
         )
         services = []
         for msg in resp:
             services.extend(BluetoothGATTServices.from_pb(msg).services)
-        return BluetoothGATTServices(address=address, services=services)
+        return ESPHomeBluetoothGATTServices(address=address, services=services)
 
     async def bluetooth_gatt_read(
-        self, address: int, characteristic_handle: int, timeout: float = 10.0
+        self,
+        address: int,
+        characteristic_handle: int,
+        timeout: float = DEFAULT_BLE_TIMEOUT,
     ) -> bytearray:
         self._check_authenticated()
 
@@ -564,7 +579,7 @@ class APIClient:
         self,
         address: int,
         handle: int,
-        timeout: float = 10.0,
+        timeout: float = DEFAULT_BLE_TIMEOUT,
     ) -> bytearray:
         self._check_authenticated()
 
@@ -629,7 +644,9 @@ class APIClient:
         )
 
         async def stop_notify() -> None:
-            assert self._connection is not None
+            if self._connection is None:
+                return
+
             self._connection.remove_message_callback(on_msg)
 
             self._check_authenticated()
