@@ -47,6 +47,8 @@ from .model import APIVersion
 
 _LOGGER = logging.getLogger(__name__)
 
+BUFFER_SIZE = 1024 * 1024  # Set buffer limit to 1MB
+
 
 @dataclass
 class ConnectionParams:
@@ -103,28 +105,47 @@ class APIConnection:
 
         self._ping_stop_event = asyncio.Event()
 
+        self._to_process: asyncio.Queue[message.Message] = asyncio.Queue()
+
+        self._process_task: Optional[asyncio.Task[None]] = None
+
+        self._connect_lock: asyncio.Lock = asyncio.Lock()
+        self._cleanup_task: Optional[asyncio.Task[None]] = None
+
     async def _cleanup(self) -> None:
         """Clean up all resources that have been allocated.
 
         Safe to call multiple times.
         """
-        if self._frame_helper is not None:
-            await self._frame_helper.close()
-            self._frame_helper = None
 
-        if self._socket is not None:
-            self._socket.close()
-            self._socket = None
+        async def _do_cleanup() -> None:
+            async with self._connect_lock:
+                if self._frame_helper is not None:
+                    await self._frame_helper.close()
+                    self._frame_helper = None
 
-        if not self._on_stop_called and self._connect_complete:
-            # Ensure on_stop is called
-            asyncio.create_task(self.on_stop())
-            self._on_stop_called = True
+                if self._process_task is not None:
+                    self._process_task.cancel()
+                    with suppress(asyncio.CancelledError):
+                        await self._process_task
+                    self._process_task = None
 
-        # Note: we don't explicitly cancel the ping/read task here
-        # That's because if not written right the ping/read task could cancel
-        # themself, effectively ending execution after _cleanup which may be unexpected
-        self._ping_stop_event.set()
+                if self._socket is not None:
+                    self._socket.close()
+                    self._socket = None
+
+                if not self._on_stop_called and self._connect_complete:
+                    # Ensure on_stop is called
+                    asyncio.create_task(self.on_stop())
+                    self._on_stop_called = True
+
+                # Note: we don't explicitly cancel the ping/read task here
+                # That's because if not written right the ping/read task could cancel
+                # themself, effectively ending execution after _cleanup which may be unexpected
+                self._ping_stop_event.set()
+
+        if not self._cleanup_task or not self._cleanup_task.done():
+            self._cleanup_task = asyncio.create_task(_do_cleanup())
 
     async def _connect_resolve_host(self) -> hr.AddrInfo:
         """Step 1 in connect process: resolve the address."""
@@ -148,6 +169,16 @@ class APIConnection:
         )
         self._socket.setblocking(False)
         self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+        # Try to reduce the pressure on esphome device as it measures
+        # ram in bytes and we measure ram in megabytes.
+        try:
+            self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, BUFFER_SIZE)
+        except OSError as err:
+            _LOGGER.warning(
+                "%s: Failed to set socket receive buffer size: %s",
+                self.log_name,
+                err,
+            )
 
         _LOGGER.debug(
             "%s: Connecting to %s:%s (%s)",
@@ -171,7 +202,9 @@ class APIConnection:
 
     async def _connect_init_frame_helper(self) -> None:
         """Step 3 in connect process: initialize the frame helper and init read loop."""
-        reader, writer = await asyncio.open_connection(sock=self._socket)
+        reader, writer = await asyncio.open_connection(
+            sock=self._socket, limit=BUFFER_SIZE
+        )  # Set buffer limit to 1MB
 
         if self._params.noise_psk is None:
             self._frame_helper = APIPlaintextFrameHelper(reader, writer)
@@ -185,11 +218,15 @@ class APIConnection:
 
         # Create read loop
         asyncio.create_task(self._read_loop())
+        # Create process loop
+        self._process_task = asyncio.create_task(self._process_loop())
 
     async def _connect_hello(self) -> None:
         """Step 4 in connect process: send hello and get api version."""
         hello = HelloRequest()
         hello.client_info = self._params.client_info
+        hello.api_version_major = 1
+        hello.api_version_minor = 7
         try:
             resp = await self.send_message_await_response(hello, HelloResponse)
         except TimeoutAPIError as err:
@@ -277,19 +314,23 @@ class APIConnection:
             if login:
                 await self.login()
 
-        try:
-            # Allow 2 minutes for connect; this is only as a last measure
-            # to protect from issues if some part of the connect process mistakenly
-            # does not have a timeout
-            async with async_timeout.timeout(120.0):
-                await _do_connect()
-        except Exception:  # pylint: disable=broad-except
-            # Always clean up the connection if an error occured during connect
-            self._connection_state = ConnectionState.CLOSED
-            await self._cleanup()
-            raise
+        # A connection lock must be created to avoid potential issues where
+        # connect has succeeded but not yet returned, followed by a disconnect.
+        # See esphome/aioesphomeapi#258 for more information
+        async with self._connect_lock:
+            try:
+                # Allow 2 minutes for connect; this is only as a last measure
+                # to protect from issues if some part of the connect process mistakenly
+                # does not have a timeout
+                async with async_timeout.timeout(120.0):
+                    await _do_connect()
+            except Exception:  # pylint: disable=broad-except
+                # Always clean up the connection if an error occured during connect
+                self._connection_state = ConnectionState.CLOSED
+                await self._cleanup()
+                raise
 
-        self._connect_complete = True
+            self._connect_complete = True
 
     async def login(self) -> None:
         """Send a login (ConnectRequest) and await the response."""
@@ -471,9 +512,19 @@ class APIConnection:
         except Exception as e:
             raise ProtocolAPIError(f"Invalid protobuf message: {e}") from e
         _LOGGER.debug("%s: Got message of type %s: %s", self.log_name, type(msg), msg)
-        for msg_handler in self._message_handlers[:]:
-            msg_handler(msg)
-        await self._handle_internal_messages(msg)
+        self._to_process.put_nowait(msg)
+
+    async def _process_loop(self) -> None:
+        while True:
+            if not self._is_socket_open:
+                # Socket closed but task isn't cancelled yet
+                break
+
+            msg = await self._to_process.get()
+
+            for handler in self._message_handlers[:]:
+                handler(msg)
+            await self._handle_internal_messages(msg)
 
     async def _read_loop(self) -> None:
         while True:
