@@ -13,6 +13,7 @@ from typing import (
     Union,
     cast,
 )
+from uuid import UUID
 
 from .util import fix_float_single_double_conversion
 
@@ -100,12 +101,13 @@ class DeviceInfo(APIModelBase):
     mac_address: str = ""
     compilation_time: str = ""
     model: str = ""
+    manufacturer: str = ""
     has_deep_sleep: bool = False
     esphome_version: str = ""
     project_name: str = ""
     project_version: str = ""
     webserver_port: int = 0
-    has_bluetooth_proxy: bool = False
+    bluetooth_proxy_version: int = 0
 
 
 class EntityCategory(APIIntEnum):
@@ -762,6 +764,11 @@ def _long_uuid(uuid: str) -> str:
     ).lower()
 
 
+def _join_split_uuid(value: List[int]) -> str:
+    """Convert a high/low uuid into a single string."""
+    return str(UUID(int=((value[0] << 64) | value[1])))
+
+
 def _convert_bluetooth_le_service_uuids(value: List[str]) -> List[str]:
     return [_long_uuid(v) for v in value]
 
@@ -772,7 +779,7 @@ def _convert_bluetooth_le_service_data(
     if isinstance(value, dict):
         return value
 
-    return {_long_uuid(v.uuid): bytes(v.data) for v in value}  # type: ignore
+    return {_long_uuid(v.uuid): bytes(v.data if v.data else v.legacy_data) for v in value}  # type: ignore
 
 
 def _convert_bluetooth_le_manufacturer_data(
@@ -780,7 +787,8 @@ def _convert_bluetooth_le_manufacturer_data(
 ) -> Dict[int, bytes]:
     if isinstance(value, dict):
         return value
-    return {int(v.uuid, 16): bytes(v.data) for v in value}  # type: ignore
+    # v.data if v.data else v.legacy_data is backwards compatable with ESPHome devices before 2022.10.0
+    return {int(v.uuid, 16): bytes(v.data if v.data else v.legacy_data) for v in value}  # type: ignore
 
 
 @dataclass(frozen=True)
@@ -813,14 +821,13 @@ class BluetoothGATTRead(APIModelBase):
     address: int = 0
     handle: int = 0
 
-    data: bytes = b""
+    data: bytes = field(default_factory=bytes)
 
 
 @dataclass(frozen=True)
 class BluetoothGATTDescriptor(APIModelBase):
-    uuid: str = converter_field(default="", converter=_long_uuid)
+    uuid: str = converter_field(default="", converter=_join_split_uuid)
     handle: int = 0
-    description: str = ""
 
     @classmethod
     def convert_list(cls, value: List[Any]) -> List["BluetoothGATTDescriptor"]:
@@ -835,7 +842,7 @@ class BluetoothGATTDescriptor(APIModelBase):
 
 @dataclass(frozen=True)
 class BluetoothGATTCharacteristic(APIModelBase):
-    uuid: str = converter_field(default="", converter=_long_uuid)
+    uuid: str = converter_field(default="", converter=_join_split_uuid)
     handle: int = 0
     properties: int = 0
 
@@ -856,7 +863,7 @@ class BluetoothGATTCharacteristic(APIModelBase):
 
 @dataclass(frozen=True)
 class BluetoothGATTService(APIModelBase):
-    uuid: str = converter_field(default="", converter=_long_uuid)
+    uuid: str = converter_field(default="", converter=_join_split_uuid)
     handle: int = 0
     characteristics: List[BluetoothGATTCharacteristic] = converter_field(
         default_factory=list, converter=BluetoothGATTCharacteristic.convert_list
@@ -881,6 +888,12 @@ class BluetoothGATTServices(APIModelBase):
     )
 
 
+@dataclass(frozen=True)
+class ESPHomeBluetoothGATTServices:
+    address: int = 0
+    services: List[BluetoothGATTService] = field(default_factory=list)
+
+
 @dataclass(frozen=True)
 class BluetoothConnectionsFree(APIModelBase):
     free: int = 0
